from nebula.graph import ttypes,GraphService
from nebula.ConnectionPool import ConnectionPool
from nebula.Client import GraphClient
from maintain_PlatoUtils.maintain_PlatoUtils import wrapNebula2Df
# from maintain_PlatoUtils import wrapNebula2Df
import tqdm
from typing import Callable, Any, Iterable
import traceback

class NodeInfo:
    def __init__(self,nodeType,nodeIdAttr,nodeIdVal=None):
        self.nodeType=nodeType
        self.nodeIdAttr=nodeIdAttr
        self.nodeIdVal=nodeIdVal
        self.nodeInfo=self.__dict__
        
        
class EdgeInfo:
    def __init__(self,edgeType,direct="",**edgeAttrKWargs):
        self.edgeType=edgeType
        self.edgeAttrDict=edgeAttrKWargs
        self.edgeInfo={
            "edgeType":edgeType,
            "direct":direct,
            **edgeAttrKWargs
        }
        
class RDFInfo:
    def __init__(self,head:NodeInfo,edge:EdgeInfo,tail:NodeInfo):
        self.head=head
        self.edge=edge
        self.tail=tail

class GraphWrapClient:
    
    def __init__(self,gHost,gPort,gUser,gPassword,gDbName=""):
        '''初始化gClient'''
        Connection_pool = ConnectionPool(gHost, gPort,network_timeout=300000)
        self.gClient = GraphClient(Connection_pool)
        self.gClient.authenticate(gUser, gPassword)
        
        if len(gDbName)>0:
            self.gClient.execute_query("USE {}".format(gDbName))
            self.setGDbName(gDbName)
        
    def setGDbName(self,gDbName):
        '''设定图空间'''
        self.gClient.execute_query("USE {}".format(gDbName))
        self.gClient.set_space(gDbName)
        return self
        
    def execute_query(self,queryStr):
        '''query操作'''
        queryReq=self.gClient.execute_query(queryStr)
        reqDf=None
        if queryReq.error_code==0 and queryReq.rows is not None:
            reqDf=wrapNebula2Df(queryReq)
        return queryReq,reqDf
    
    def delVertex(self,sysIdList,delRel=True):
        '''（关联）删除节点'''
        errCode=0
        if delRel==True:
            showEdgeReq=self.gClient.execute_query("SHOW EDGES")
            errCode+=showEdgeReq.error_code
            
            relDf=wrapNebula2Df(showEdgeReq)
            if relDf.shape[0]>0:
                relList=relDf["Name"].values.flatten().tolist()
                for relItem in tqdm.tqdm(relList,desc="del edges with vertexes"):
                    for srcSysIdItem in tqdm.tqdm(sysIdList,desc="del srcs from edges with vertexes"):
                        relTailStr="GO FROM {srcSysId} OVER {edgeName} BIDIRECT YIELD {edgeName}._dst AS tgtSysId".format(
                            srcSysId=srcSysIdItem,
                            edgeName=relItem)
                        # print(relTailStr)
                        relTailReq=self.gClient.execute_query(relTailStr)
                        errCode+=relTailReq.error_code
                        relTailSysIdDf=wrapNebula2Df(relTailReq)
                        if relTailSysIdDf.shape[0]>0:
                            relTailSysIdList=relTailSysIdDf["tgtSysId"].values.flatten().tolist()
                            delOrderGroupStr=",".join(["{}->{}".format(srcSysIdItem,tailSysIdItem) for tailSysIdItem in relTailSysIdList])
                            delReverseGroupStr=",".join(["{}->{}".format(tailSysIdItem,srcSysIdItem) for tailSysIdItem in relTailSysIdList])
                            delGroupStr=",".join([delOrderGroupStr,delReverseGroupStr])
                            delStr="DELETE EDGE {} {}".format(relItem,delGroupStr)
                            # print(delStr)
                            delReq=self.gClient.execute_query(delStr)
                            errCode+=delReq.error_code
        return {"error_code":errCode}
        
        
class GraphWrapQuery:
    
    def __init__(self):
        self.yieldAttrList=[]
        self.yieldSysIdList=[]
    
    def singleSearchFunc_wait(self,hetDict,head=True,singleYieldList={}):
        args=[hetDict]
        kwargs={
            "head":head,
            "singleYieldList":singleYieldList
        }
        return self.singleSearchFunc,args,kwargs
    
    def singleSearchFunc(self,hetDict,head=True,singleYieldList={}):
        '''
        direction: 默认单项
        '''
        
        singleQueryStrList=[]
        
        if type(hetDict)==dict:
            headInfo=hetDict.get("head",{})
            edgeInfo=hetDict.get("edge",{})
            tailInfo=hetDict.get("tail",{})
        elif type(hetDict)==RDFInfo:
            headInfo,edgeInfo,tailInfo=hetDict.head.nodeInfo,hetDict.edge.edgeInfo,hetDict.tail.nodeInfo
        elif type(hetDict)==NodeInfo:
            headInfo=hetDict.nodeInfo
            edgeInfo={}
            tailInfo={}
        
        if "direct" not in edgeInfo and len(edgeInfo)>0:
            edgeInfo["direct"]=""
        
        singleYieldDict={}
        if len(singleYieldList)>0:
            for nodeTypeAttrItem in singleYieldList:
                if type(nodeTypeAttrItem["nodeAttr"])==str:
                    nodeTypeAttrItem["nodeAttr"]=nodeTypeAttrItem["nodeAttr"].strip()
                singleYieldDict[nodeTypeAttrItem["nodeType"]]=singleYieldDict.get(nodeTypeAttrItem["nodeType"],[])+[nodeTypeAttrItem["nodeAttr"]]
            
        # LOOKUP语句构建
        if head==True:  
            singleQueryStrList=["LOOKUP ON {nodeType} WHERE {nodeType}.{nodeIdAttr}=='{nodeIdVal}'|\
                                        YIELD $-.VertexID AS src{nodeType}SysId".format(
                                                                                            nodeType=headInfo["nodeType"],
                                                                                            nodeIdAttr=headInfo["nodeIdAttr"],
                                                                                            nodeIdVal=headInfo["nodeIdVal"]
                                                                                        )]
            srcSysIdName="src{}SysId".format(headInfo["nodeType"])
            self.yieldSysIdList.append(srcSysIdName)
        
        # yield语句初始化
        yieldAttrList=[]
        pureAttrYieldList=[]
        for singleYieldTypeKey in singleYieldDict:
            for singleYieldAttrVal in singleYieldDict[singleYieldTypeKey]:
                nodeType=singleYieldTypeKey
                nodeAttr=singleYieldAttrVal
                nodeTypeAttrName="{nodeType}{nodeAttr}".format(nodeType=nodeType,
                                                            nodeAttr=nodeAttr)
                if nodeTypeAttrName not in self.yieldAttrList:
                    if len(headInfo)>0 and headInfo["nodeType"]==singleYieldTypeKey:
                        yieldAttrList.append("$^.{nodeType}.{nodeAttr} AS {nodeType}{nodeAttr}".format(nodeType=nodeType,
                                                                                                    nodeAttr=nodeAttr))
                        if nodeTypeAttrItem not in self.yieldAttrList:
                            pureAttrYieldList.append(nodeTypeAttrName)
                    if len(edgeInfo)>0 and tailInfo["nodeType"]==singleYieldTypeKey:
                        yieldAttrList.append("$$.{nodeType}.{nodeAttr} AS {nodeType}{nodeAttr}".format(nodeType=nodeType,
                                                                                                    nodeAttr=nodeAttr))
                        if nodeTypeAttrItem not in self.yieldAttrList:
                            pureAttrYieldList.append(nodeTypeAttrName)
                        
        oldYieldAttrList=[]
        for yieldAttrItem in self.yieldAttrList:
            oldYieldAttrList.append("$-.{oldYieldAttr} AS {oldYieldAttr}".format(oldYieldAttr=yieldAttrItem))
        yieldAttrListStr=""
        if len(yieldAttrList)>0:
            yieldAttrListStr=","+",".join(yieldAttrList+oldYieldAttrList)

        yieldSysIdList=["$-.{yieldSysId} AS {yieldSysId}".format(yieldSysId=yieldSysIdItem) for yieldSysIdItem in self.yieldSysIdList]
        yieldSysIdListStr=""
        if len(yieldSysIdList)>0:
            yieldSysIdListStr=","+",".join(yieldSysIdList) 
        
        # GO语句构建-WHERE语句
        
        
        # GO语句构建-完整化
        if len(edgeInfo)>0:
            # RDF搜索
            if len(tailInfo)>0:
                startSysId=self.yieldSysIdList[-1]
                singleQueryStrList.append("GO FROM $-.{startSysId} OVER {edgeType} {direct} YIELD {edgeType}._dst AS tgt{nodeType}SysId {yieldList}{yieldAttrList}".format(
                                                        startSysId=startSysId,
                                                        edgeType=edgeInfo["edgeType"],
                                                        direct=edgeInfo["direct"],
                                                        nodeType=tailInfo["nodeType"],
                                                        yieldList=yieldSysIdListStr,
                                                        yieldAttrList=yieldAttrListStr
                                                    ))
            else:
                singleQueryStrList.append("GO FROM $-.VertexID OVER {edgeType} {direct} YIELD {edgeType}._dst AS tgt{nodeType}SysId {yieldList}{yieldAttrList}".format(
                                                        edgeType=edgeInfo["edgeType"],
                                                        direct=edgeInfo["direct"],
                                                        nodeType=tailInfo["nodeType"],
                                                        yieldList=yieldSysIdListStr,
                                                        yieldAttrList=yieldAttrListStr
                                                    ))
        
            # 整理新的yield列表
            tgtSysIdName="tgt{}SysId".format(tailInfo["nodeType"])
            self.yieldSysIdList.append(tgtSysIdName)
            self.yieldAttrList+=pureAttrYieldList
            # self.yieldAttrList=list(set(self.yieldAttrList))
        else:
            # 单节点搜索
            fetchStr="FETCH PROP ON {nodeType} $-.{srcSysID}".format(nodeType=headInfo["nodeType"],srcSysID=srcSysIdName)
            singleQueryStrList.append(fetchStr)
            
        
        totalQuery="|".join(singleQueryStrList)
        
        return totalQuery
            
    def intersectSearchFunc_wait(self,*queryList):
        return self.intersectSearchFunc,queryList
        
    def intersectSearchFunc(self,*queryFuncList):
        tmpQueryStrList=[]
        for queryFuncGroupItem in queryFuncList:
            self.yieldSysIdList=[]
            self.yieldAttrList=[]
            if type(queryFuncGroupItem)==str:
                # 返回query
                queryItem=queryFuncGroupItem
            else:
                # 返回函数
                if len(queryFuncGroupItem)==3:
                    queryFunc,args,kwargs=queryFuncGroupItem
                    queryItem=queryFunc(*args,**kwargs)
                else:
                    queryFunc,args=queryFuncGroupItem
                    queryItem=queryFunc(*args)
            tmpQueryStrList.append("({})".format(queryItem))
        queryStr=" INTERSECT ".join(tmpQueryStrList)
        return queryStr
              
    def unionSearchFunc_wait(self,*queryList):
        return self.unionSearchFunc,queryList
        
    def unionSearchFunc(self,*queryFuncList):
        tmpQueryStrList=[]
        for queryFuncGroupItem in queryFuncList:
            self.yieldSysIdList=[]
            self.yieldAttrList=[]
            if type(queryFuncGroupItem)==str:
                queryItem=queryFuncGroupItem
            else:
                if len(queryFuncGroupItem)==3:
                    queryFunc,args,kwargs=queryFuncGroupItem
                    queryItem=queryFunc(*args,**kwargs)
                else:
                    queryFunc,args=queryFuncGroupItem
                    queryItem=queryFunc(*args)
            tmpQueryStrList.append("({})".format(queryItem))
        queryStr=" UNION ".join(tmpQueryStrList)
        return queryStr
        
    def seqSearchFunc_wait(self,*queryList):
        for queryItem in queryList[1:]:
            queryItem[2]["head"]=False
        return self.seqSearchFunc,queryList
        
    def seqSearchFunc(self,*queryGroupList):
        '''第一个singleFunc的head为True'''
        tmpQueryStrList=[]
        for queryGroupItem in queryGroupList:
            
            if type(queryGroupItem)==str:
                queryItem=queryGroupItem
            else:
                if len(queryGroupItem)==3:
                    queryFuncItem,args,kwargs=queryGroupItem
                    queryItem=queryFuncItem(*args,**kwargs)
                else:
                    queryFuncItem,args=queryGroupItem
                    queryItem=queryFuncItem(*args)
                
            tmpQueryStrList.append(queryItem)
        queryStr="|".join(tmpQueryStrList)
        return queryStr
        
    def renew(self):
        self.yieldAttrList=[]
        self.yieldSysIdList=[]
        
    def runWaitFunc(self,waitFuncGroupList,maxLayer=0):
        if type(waitFuncGroupList)==list or type(waitFuncGroupList)==dict:
            return waitFuncGroupList
        typeList=[type(waitFuncGroupItem).__name__ for waitFuncGroupItem in waitFuncGroupList]
        typeSetList=list(set(typeList))
        if "str" not in typeList:
            if type(waitFuncGroupList[0]).__name__=="method":
                if waitFuncGroupList[0].__name__=="singleSearchFunc":
                    print("singleSearchFunc")
                    if maxLayer==0:
                        returnVal=waitFuncGroupList[0](*waitFuncGroupList[1],**waitFuncGroupList[2])
                        return returnVal
                    else:
                        return waitFuncGroupList
                elif waitFuncGroupList[0].__name__=="seqSearchFunc":
                    print("seqSearchFunc")
                    returnVal=waitFuncGroupList[0](*self.runWaitFunc(waitFuncGroupList[1],maxLayer=maxLayer+1))
                    return returnVal
                elif waitFuncGroupList[0].__name__=="unionSearchFunc":
                    print("unionSearchFunc")
                    returnVal=waitFuncGroupList[0](*self.runWaitFunc(waitFuncGroupList[1],maxLayer=maxLayer+1))
                    return returnVal
                elif waitFuncGroupList[0].__name__=="intersectSearchFunc":
                    print("intersectSearchFunc")
                    returnVal=waitFuncGroupList[0](*self.runWaitFunc(waitFuncGroupList[1],maxLayer=maxLayer+1))
                    return returnVal
                
                
                # if type(waitFuncGroupList[1])==tuple:
                #     try:
                #         return waitFuncGroupList[0](*self.runWaitFunc(waitFuncGroupList[1]))
                #     except Exception as ex:
                #         traceback.print_exc()
                #         print(ex)
                # else:
                #     try:
                #         return waitFuncGroupList[0](*self.runWaitFunc(waitFuncGroupList[1]),**self.runWaitFunc(waitFuncGroupList[2]))
                #     except Exception as ex:
                #         traceback.print_exc()
                #         print(ex)
            # elif len(typeSetList)==1 and typeSetList[0]=="tuple":
            #     returnVal=[self.runWaitFunc(waitFuncGroupItem) for waitFuncGroupItem in waitFuncGroupList]
            #     return returnVal
            else:
                return waitFuncGroupList
            
    def transMethod(self,methodName):
        if methodName=="singleSearch":
            return self.singleSearchFunc_wait
        elif methodName=="seqSearch":
            return self.seqSearchFunc_wait
        elif methodName=="unionSearch":
            return self.unionSearchFunc_wait
        elif methodName=="intersectSearch":
            return self.intersectSearchFunc_wait
            
    def wrapJson2Query(self,myQueryJson):
        if myQueryJson["searchMethod"] in ["seqSearch","unionSearch","intersectSearch"]:
            searchMethod=self.transMethod(myQueryJson["searchMethod"])
            searchFrameList=myQueryJson["searchFrame"]
            return searchMethod(*[self.wrapJson2Query(searchFrameItem) for searchFrameItem in searchFrameList])
        if myQueryJson["searchMethod"] in ["singleSearch"]:
            searchMethod=self.transMethod(myQueryJson["searchMethod"])
            searchFrameDict=myQueryJson["searchFrame"]
            searchFrameDict["singleYieldList"]=searchFrameDict.get("singleYieldList",{})
            return searchMethod(searchFrameDict,singleYieldList=searchFrameDict["singleYieldList"])
            
        
if __name__=="__main__":
    
    gHost="9.135.95.249"
    gPort=13708
    gUser="root"
    gPassword="nebula"
    gDbName="company_product_field_musklin"

    myGClient=GraphWrapClient(gHost,gPort,gUser,gPassword,gDbName=gDbName)
    myGQuery=GraphWrapQuery()
    
    # # single vertex search with singleSearch
    # queryStr=myGQuery.singleSearchFunc(NodeInfo("Company","CompanyName",nodeIdVal="深圳市腾讯计算机系统有限公司"),
    #                                singleYieldList=[{"nodeType":"Company","nodeAttr":"CompanyName"}])
    # myGQuery.renew()
    
    # # singleSearch
    # queryStr=myGQuery.singleSearchFunc(RDFInfo(NodeInfo("Company","CompanyName",nodeIdVal="深圳市腾讯计算机系统有限公司"),
    #                                        EdgeInfo("produce"),
    #                                        NodeInfo("Product","ProductName",nodeIdVal=None)),
    #                                singleYieldList=[{"nodeType":"Company","nodeAttr":"CompanyName"},
    #                                                 {"nodeType":"Product","nodeAttr":"ProductName"}])
    # myGQuery.renew()
    
    # # seqSearch
    # queryStr=myGQuery.seqSearchFunc(
    #     myGQuery.singleSearchFunc_wait(RDFInfo(NodeInfo("Company","CompanyName",nodeIdVal="深圳市腾讯计算机系统有限公司"),
    #                                        EdgeInfo("produce"),
    #                                        NodeInfo("Product","ProductName",nodeIdVal=None)),
    #                                singleYieldList=[{"nodeType":"Company","nodeAttr":"CompanyName"},
    #                                                 {"nodeType":"Product","nodeAttr":"ProductName"}]),
    #     myGQuery.singleSearchFunc_wait(RDFInfo(NodeInfo("Product","ProductName",nodeIdVal=None),
    #                                        EdgeInfo("belongTo"),
    #                                        NodeInfo("Field","FieldName",nodeIdVal=None)),
    #                                singleYieldList=[{"nodeType":"Product","nodeAttr":"ProductName"},
    #                                                 {"nodeType":"Field","nodeAttr":"FieldName"}],
    #                                head=False)
    # )
    # myGQuery.renew()
    
    # # unionSearch
    # queryStr=myGQuery.unionSearchFunc(
    #     myGQuery.singleSearchFunc_wait(RDFInfo(NodeInfo("Company","CompanyName",nodeIdVal="深圳市腾讯计算机系统有限公司"),
    #                                         EdgeInfo("produce"),
    #                                         NodeInfo("Product","ProductName",nodeIdVal=None)),
    #                                 singleYieldList=[{"nodeType":"Company","nodeAttr":"CompanyName"},
    #                                                 {"nodeType":"Product","nodeAttr":"ProductName"}]),
    #     myGQuery.singleSearchFunc_wait(RDFInfo(NodeInfo("Company","CompanyName",nodeIdVal="国泰君安期货有限公司"),
    #                                         EdgeInfo("produce"),
    #                                         NodeInfo("Product","ProductName",nodeIdVal=None)),
    #                                 singleYieldList=[{"nodeType":"Company","nodeAttr":"CompanyName"},
    #                                                 {"nodeType":"Product","nodeAttr":"ProductName"}])
    # )
    # myGQuery.renew()
    
    # # union+seqSearch
    # queryFunc=myGQuery.unionSearchFunc_wait(
    #     myGQuery.seqSearchFunc_wait(
    #         myGQuery.singleSearchFunc_wait(RDFInfo(NodeInfo("Company","CompanyName",nodeIdVal="深圳市腾讯计算机系统有限公司"),
    #                                         EdgeInfo("produce"),
    #                                         NodeInfo("Product","ProductName",nodeIdVal=None)),
    #                                 singleYieldList=[{"nodeType":"Company","nodeAttr":"CompanyName"},
    #                                                     {"nodeType":"Product","nodeAttr":"ProductName"}]),
    #         myGQuery.singleSearchFunc_wait(RDFInfo(NodeInfo("Product","ProductName",nodeIdVal=None),
    #                                         EdgeInfo("belongTo"),
    #                                         NodeInfo("Field","FieldName",nodeIdVal=None)),
    #                                 singleYieldList=[{"nodeType":"Product","nodeAttr":"ProductName"},
    #                                                     {"nodeType":"Field","nodeAttr":"FieldName"}],
    #                                 head=False)
    #     ),
    #     myGQuery.seqSearchFunc_wait(
    #         myGQuery.singleSearchFunc_wait(RDFInfo(NodeInfo("Company","CompanyName",nodeIdVal="阿里巴巴"),
    #                                         EdgeInfo("produce"),
    #                                         NodeInfo("Product","ProductName",nodeIdVal=None)),
    #                                 singleYieldList=[{"nodeType":"Company","nodeAttr":"CompanyName"},
    #                                                     {"nodeType":"Product","nodeAttr":"ProductName"}]),
    #         myGQuery.singleSearchFunc_wait(RDFInfo(NodeInfo("Product","ProductName",nodeIdVal=None),
    #                                         EdgeInfo("belongTo"),
    #                                         NodeInfo("Field","FieldName",nodeIdVal=None)),
    #                                 singleYieldList=[{"nodeType":"Product","nodeAttr":"ProductName"},
    #                                                     {"nodeType":"Field","nodeAttr":"FieldName"}],
    #                                 head=False)
    #     )
    # )
    # myGQuery.renew()
    
    # # intersect+seqSearch
    # queryFunc=myGQuery.intersectSearchFunc_wait(
    #     myGQuery.seqSearchFunc_wait(
    #         myGQuery.singleSearchFunc_wait(RDFInfo(NodeInfo("Company","CompanyName",nodeIdVal="深圳市腾讯计算机系统有限公司"),
    #                                         EdgeInfo("produce"),
    #                                         NodeInfo("Product","ProductName",nodeIdVal=None)),
    #                                 singleYieldList=[{"nodeType":"Company","nodeAttr":"CompanyName"},
    #                                                     {"nodeType":"Product","nodeAttr":"ProductName"}]),
    #         myGQuery.singleSearchFunc_wait(RDFInfo(NodeInfo("Product","ProductName",nodeIdVal=None),
    #                                         EdgeInfo("belongTo"),
    #                                         NodeInfo("Field","FieldName",nodeIdVal=None)),
    #                                 singleYieldList=[{"nodeType":"Product","nodeAttr":"ProductName"},
    #                                                     {"nodeType":"Field","nodeAttr":"FieldName"}],
    #                                 head=False)
    #     ),
    #     myGQuery.seqSearchFunc_wait(
    #         myGQuery.singleSearchFunc_wait(RDFInfo(NodeInfo("Company","CompanyName",nodeIdVal="阿里巴巴"),
    #                                         EdgeInfo("produce"),
    #                                         NodeInfo("Product","ProductName",nodeIdVal=None)),
    #                                 singleYieldList=[{"nodeType":"Company","nodeAttr":"CompanyName"},
    #                                                     {"nodeType":"Product","nodeAttr":"ProductName"}]),
    #         myGQuery.singleSearchFunc_wait(RDFInfo(NodeInfo("Product","ProductName",nodeIdVal=None),
    #                                         EdgeInfo("belongTo"),
    #                                         NodeInfo("Field","FieldName",nodeIdVal=None)),
    #                                 singleYieldList=[{"nodeType":"Product","nodeAttr":"ProductName"},
    #                                                     {"nodeType":"Field","nodeAttr":"FieldName"}],
    #                                 head=False)
    #     )
    # )
    # myGQuery.renew()
    
    # # wrapJson2Qeury-1
    # queryJson={
    #         "searchMethod":"seqSearch",
    #         "searchFrame":[
    #             {
    #                 "searchMethod":"singleSearch",
    #                 "searchFrame":{
    #                     "head":{
    #                         "nodeType":"Company",
    #                         "nodeIdAttr":"CompanyName",
    #                         "nodeIdVal":"深圳市腾讯计算机系统有限公司"
    #                     },
    #                     "edge":{
    #                         "edgeType":"produce"
    #                     },
    #                     "tail":{
    #                         "nodeType":"Product",
    #                         "nodeIdAttr":"ProductName",
    #                         "nodeIdVal":None
    #                     },
    #                     "singleYieldList":[
    #                         {"nodeType":"Company","nodeAttr":"CompanyName"},
    #                         {"nodeType":"Product","nodeAttr":"ProductName"}
    #                     ]
    #                 }
    #             },{
    #                 "searchMethod":"singleSearch",
    #                 "searchFrame":{
    #                     "head":{
    #                         "nodeType":"Product",
    #                         "nodeIdAttr":"ProductName",
    #                         "nodeIdVal":None
    #                     },
    #                     "edge":{
    #                         "edgeType":"belongTo"
    #                     },
    #                     "tail":{
    #                         "nodeType":"Field",
    #                         "nodeIdAttr":"FieldName",
    #                         "nodeIdVal":None
    #                     },
    #                     "singleYieldList":[
    #                         {"nodeType":"Company","nodeAttr":"CompanyName"},
    #                         {"nodeType":"Product","nodeAttr":"ProductName"},
    #                         {"nodeType":"Field","nodeAttr":"FieldName"}
    #                     ]
    #                 }
    #             }
    #         ]
    #     }
    # queryFunc=myGQuery.wrapJson2Query(queryJson)
    # queryStr=myGQuery.runWaitFunc(queryFunc)
    
    # wrapJson2Qeury-2
    queryJson={
        "searchMethod":"singleSearch",
        "searchFrame":{
            "head":{
                "nodeType":"Company",
                "nodeIdAttr":"CompanyName",
                "nodeIdVal":"深圳市腾讯计算机系统有限公司"
            },
            "edge":{
                "edgeType":"produce"
            },
            "tail":{
                "nodeType":"Product",
                "nodeIdAttr":"ProductName",
                "nodeIdVal":None
            },
            "singleYieldList":[
                {"nodeType":"Company","nodeAttr":"CompanyName"},
                {"nodeType":"Product","nodeAttr":"ProductName"}
            ]
        }
    }
    queryFunc=myGQuery.wrapJson2Query(queryJson)
    queryStr=myGQuery.runWaitFunc(queryFunc)
    
    
    print(queryStr)
    print(123)
    # queryReq,queryDf=myGClient.execute_query("INSERT VERTEX hv1NodeType(headIdAttr) VALUES uuid('hv1NodeType_7'):(7)")
    # myGClient.singleFunc(hetDict=)


from nebula.graph import ttypes,GraphService
from nebula.ConnectionPool import ConnectionPool
from nebula.Client import GraphClient
from maintain_PlatoUtils.maintain_PlatoUtils import wrapNebula2Df,pdPlatoTypeSame
import pandas as pd
import json
import os
import requests

def buildVertex(vertexJson,graphClient,nodeSysIDUUID=True):

    vertexJsonList=[]
    for vertexInfoItem in vertexJson:
        vertexPath=vertexInfoItem["file_path"]
        vertexFileName=vertexInfoItem["file_name"].split(".")[0]+"_copy.csv"
        csv2plato_attr_map=vertexInfoItem["csv2plato_attr_map"]
        csv_attr2type_map=vertexInfoItem["attr_type_map"]
        nodeType=vertexInfoItem["node_type"]
        idCol=vertexInfoItem["id_col"]
        vertexDf=pd.read_csv(vertexPath)
        
        gNodeAttrDf=wrapNebula2Df(graphClient.execute_query("DESCRIBE TAG {}".format(nodeType)))
        
        attrTypeList=[]
        if gNodeAttrDf.shape[0]>0:
            gNodeAttrList=gNodeAttrDf["Field"].values.flatten().tolist()
            
            for attrKeyItem in csv_attr2type_map:
                if attrKeyItem in  csv2plato_attr_map and csv2plato_attr_map[attrKeyItem] in gNodeAttrList:
                    if attrKeyItem==idCol:
                        attrTypeList.append({
                                "name": csv2plato_attr_map[attrKeyItem],
                                "type": "string",
                                "index": list(vertexDf.columns).index(attrKeyItem)
                            })
                    else:
                        if pdPlatoTypeSame(vertexDf[attrKeyItem],csv_attr2type_map[attrKeyItem]):
                            attrTypeList.append({
                                    "name": csv2plato_attr_map[attrKeyItem],
                                    "type": csv_attr2type_map[attrKeyItem],
                                    "index": list(vertexDf.columns).index(attrKeyItem)
                                })
                        elif csv_attr2type_map[attrKeyItem]=="string":
                            vertexDf[attrKeyItem]=vertexDf[attrKeyItem].fillna("null")
                            vertexDf[attrKeyItem]=vertexDf[attrKeyItem].astype(str)
                            attrTypeList.append({
                                    "name": csv2plato_attr_map[attrKeyItem],
                                    "type": csv_attr2type_map[attrKeyItem],
                                    "index": list(vertexDf.columns).index(attrKeyItem)
                                })
                        else:
                            raise TypeError("csv({}_{})与uploadSchema.json类型冲突，请核查".format(vertexPath.split("/")[-1],
                                                                                            attrKeyItem))
            uuidFunDict={"function":"uuid"} if nodeSysIDUUID else {}
            vertexItemJson={
                        "path": "/upload-dir/{}".format(vertexFileName),
                        "failDataPath": "/upload-dir/tmp/err/{}Fail.csv".format(vertexFileName.split(".")[0]),
                        "batchSize": 10,
                        "type": "csv",
                        "csv": {
                            "withHeader": False,
                            "withLabel": False
                        },
                        "schema": {
                            "type": "vertex",
                            "vertex": {
                                "vid": {**{
                                    "index": list(vertexDf.columns).index(idCol)
                                },**uuidFunDict},
                                "tags": [
                                    {
                                        "name": nodeType,
                                        "props": attrTypeList
                                    }
                                ]
                            }
                        }
                    }
            vertexJsonList.append(vertexItemJson)
    return vertexJsonList
    
def buildEdge(edgeJson,graphClient,srcSysIDUUID=True,tgtSysIDUUID=True):

    edgeJsonList=[]
    for edgeInfoItem in edgeJson:
        edgePath=edgeInfoItem["file_path"]
        edgeFileName=edgeInfoItem["file_name"].split(".")[0]+"_copy.csv"
        edgeType=edgeInfoItem["edge_type"]
        srcId=edgeInfoItem["src_id"]
        tgtId=edgeInfoItem["tgt_id"]
        csv2plato_attr_map=edgeInfoItem["csv2plato_attr_map"]
        csv_attr2type_map=edgeInfoItem["attr_type_map"]
        edgeDf=pd.read_csv(edgePath)
        
        gEdgeAttrDf=wrapNebula2Df(graphClient.execute_query("DESCRIBE EDGE {}".format(edgeType)))
        
        attrTypeList=[]
        if gEdgeAttrDf.shape[0]>0:
            gEdgeAttrList=gEdgeAttrDf["Field"].values.flatten().tolist()
            for attrKeyItem in csv_attr2type_map:
                if attrKeyItem in  csv2plato_attr_map and csv2plato_attr_map[attrKeyItem] in gEdgeAttrList:
                    if pdPlatoTypeSame(edgeDf[attrKeyItem],csv_attr2type_map[attrKeyItem]):
                        attrTypeList.append({
                                "name": csv2plato_attr_map[attrKeyItem],
                                "type": csv_attr2type_map[attrKeyItem],
                                "index": list(edgeDf.columns).index(attrKeyItem)
                            })
                    else:
                        raise TypeError("csv({}_{})与uploadSchema.json类型冲突，请核查".format(edgePath.split("/")[-1],
                                                                                                attrKeyItem))

        srcVIDDict={"function": "uuid"} if srcSysIDUUID else {}
        tgtVIDDict={"function": "uuid"} if tgtSysIDUUID else {}
        edgeItemJson={
                        "path": "/upload-dir/{}".format(edgeFileName),
                        "failDataPath": "/upload-dir/tmp/err/{}Fail.csv".format(edgeFileName.split(".")[0]),
                        "batchSize": 10,
                        "type": "csv",
                        "csv": {
                            "withHeader": False,
                            "withLabel": False
                        },
                        "schema": {
                            "type": "edge",
                            "edge": {
                                "name": edgeType,
                                "srcVID": {**{
                                    "index": list(edgeDf.columns).index(srcId)
                                },**srcVIDDict},
                                "dstVID": {**{
                                    "index": list(edgeDf.columns).index(tgtId)
                                },**tgtVIDDict},
                                "withRanking": False,
                                "props": attrTypeList
                            }
                        }
                    }
        edgeJsonList.append(edgeItemJson)
    return edgeJsonList

attrTypeNullDict={
    "string":"'null'",
    "int":0,
    "double":0.0
}
def createSchemaFromSchemaJson(schemaJson,graphClient):
    '''
    根据schemaJson构建图数据库/节点-边schema/节点index
    '''
    gDbName=schemaJson["gDbName"]
    graphClient.execute_query("CREATE SPACE IF NOT EXISTS {}".format(gDbName)) # 构建图数据库
    graphClient.execute_query("USE {}".format(gDbName))
    createTagErrCode=0
    createTagIndexErrCode=0
    rebuildTagIndexErrCode=0
    createEdgeErrCode=0
    if "vertex" in schemaJson: # 构建节点schema
        schemaTagList=schemaJson["vertex"]
        for schemaTagItem in schemaTagList:
            nodeType=schemaTagItem["node_type"]
            nodeIdAttr=schemaTagItem["old_id_col"]
            nodeAttrStr=",".join([" ".join([attrKey,schemaTagItem["attr_type_map"][attrKey]])+" DEFAULT {}".format(attrTypeNullDict[schemaTagItem["attr_type_map"][attrKey]]) 
                                    for attrKey in schemaTagItem["attr_type_map"]])
            
            createTagStr="CREATE TAG IF NOT EXISTS {}({})".format(nodeType,nodeAttrStr)
            createTagReq=graphClient.execute_query(createTagStr)
            createTagErrCode=createTagReq.error_code

            createTagIndexStr="CREATE TAG INDEX IF NOT EXISTS {nodeTypeLower}_{nodeTypeLower}{nodeIdAttrLower}_index ON {nodeType}({nodeIdAttr})".format(
                nodeTypeLower=nodeType.lower(),
                nodeIdAttrLower=nodeIdAttr.lower(),
                nodeType=nodeType,
                nodeIdAttr=nodeIdAttr
            )
            createTagIndexReq=graphClient.execute_query(createTagIndexStr)
            createTagIndexErrCode=createTagIndexReq.error_code
            
            rebuildTagIndexStr="REBUILD TAG INDEX {nodeTypeLower}_{nodeTypeLower}{nodeIdAttrLower}_index OFFLINE".format(
                nodeTypeLower=nodeType.lower(),
                nodeIdAttrLower=nodeIdAttr.lower(),
                nodeType=nodeType,
                nodeIdAttr=nodeIdAttr
            )
            rebuildTagIndexReq=graphClient.execute_query(rebuildTagIndexStr)
            rebuildTagIndexErrCode=rebuildTagIndexReq.error_code

    if "edge" in schemaJson: # 构建关系schema
        schemaEdgeList=schemaJson["edge"]
        for schemaEdgeItem in schemaEdgeList:
            edgeType=schemaEdgeItem["edge_type"]
            edgeAttrStr=",".join([" ".join([attrKey,schemaEdgeItem["attr_type_map"][attrKey]])+" DEFAULT {}".format(attrTypeNullDict[schemaEdgeItem["attr_type_map"][attrKey]]) 
                                    for attrKey in schemaEdgeItem["attr_type_map"]])

            createEdgeStr="CREATE EDGE IF NOT EXISTS {}({})".format(edgeType,edgeAttrStr)
            createEdgeReq=graphClient.execute_query(createEdgeStr)
            createEdgeErrCode=createEdgeReq.error_code

    return createTagErrCode,createTagIndexErrCode,rebuildTagIndexErrCode,createEdgeErrCode
            

def submitSchema(schemaJson,gUrl="http://9.135.95.249:7001",cookie=""):

    url = gUrl+"/api-import/submit"

    data = json.dumps(schemaJson)
    headers = {
        'Proxy-Connection': 'keep-alive',
        'Accept': 'application/json, text/plain, */*',
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.107 Safari/537.36 Edg/92.0.902.62',
        'Content-Type': 'application/json',
        'Origin': gUrl,
        'Referer': '{}/import'.format(gUrl),
        'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6',
        'Cookie': cookie
    }

    response = requests.request("POST", url, headers=headers, data=data)

    print(response.text)
    return response.json()


if __name__=="__main__":
    
    # test
    ghost="9.135.95.249"
    gport=13708
    guser="root"
    gpassword="nebula"
    gspace="post_skill_school_ianxu"
    gUrl="http://9.135.95.249:7001"

    schemaPath="csv2platodb/attr2Vertex_1629250000"
    
    # product
    # ghost="10.99.218.40"
    # gport=8080
    # guser="root"
    # gpassword="nebula"
    # gspace="testImport"
    # gUrl="http://10.99.218.40:8081"
    
    '''
    uploadSchema形状：
    {
        "vertex":[
            {
                "file_path": "xxx" (node),
                "file_name": "xxx.csv" (node file),
                "node_type": "NodeType",
                "id_col": "nodeID_col",
                "csv2plato_attr_map":{
                    "csvAttr":"graphAttr"
                },
                "attr_type_map": {
                    "csvAttr":"graphAttrType" (double/int/string only)
                }
            },
            ......
        ],
        "edge":[
            {
                "file_path": "xxx" (edge),
                "file_name": "xxx.csv" (edge file),
                "edge_type": "edgeType",
                "src_id": "src_node_ID_col",
                "tgt_id": "tgt_node_ID_col",
                "csv2plato_attr_map": {
                    "csvAttr":"graphAttr"
                },
                "attr_type_map": {
                    "csvAttr":"graphAttrType" (double/int/string only)
                }
            },
            ......
        ]
    }
    '''
    with open(os.path.join(schemaPath,"uploadSchema.json")) as uploadSchemaFile:
        uploadSchemaJson=json.load(uploadSchemaFile)
    
    gConnection_pool = ConnectionPool(ghost, gport,network_timeout=300000)
    gClient = GraphClient(gConnection_pool)
    gClient.authenticate(guser, gpassword)
    gClient.execute_query("USE {}".format(gspace))
    
    vertexJsonList=[]
    edgeJsonList=[]
    if "vertex" in uploadSchemaJson:
        vertexJsonList=buildVertex(uploadSchemaJson["vertex"],graphClient=gClient)
    if "edge" in uploadSchemaJson:
        edgeJsonList=buildEdge(uploadSchemaJson["edge"],graphClient=gClient)
    
    schemaJson={
        "version": "v1rc1",
        "description": "web console import",
        "clientSettings": {
            "concurrency": 10,
            "channelBufferSize": 128,
            "space": gspace,
            "connection": {
                "user": guser,
                "password": gpassword,
                "address": "{}:{}".format(ghost,gport)
            }
        },
        "logPath": "/upload-dir/tmp/import.log",
        "files": vertexJsonList+edgeJsonList
    }
    with open(os.path.join(schemaPath,"schemaJson.json"),"w+") as schemaJsonFile:
        json.dump(schemaJson,schemaJsonFile)
        
    resDict=submitSchema(schemaJson,gUrl=gUrl)
    
    print(resDict)


import requests
import os
import time

def uploadFile(fileDict,platoServerAPI):
    '''
    fileDict:{"csv":["path/xxx1.csv","path/xxx2.csv",...],"json":"xxx.json"}
    platoServerAPI:ip:port
    '''

    url = "http://{}/csv2platodb/upload".format(platoServerAPI)

    csvFileList=[
        ('csv',(csvFilePathItem.split("/")[-1],open(csvFilePathItem,'rb'),'text/csv'))
    for csvFilePathItem in fileDict["csv"]]
    jsonFileList=[
        ('json',(fileDict["json"].split("/")[-1],open(fileDict["json"],'rb'),'application/json'))
    ]
    files=csvFileList+jsonFileList

    response = requests.request("POST", url, files=files)

    print(response.text)



import pandas as pd
import json
import tqdm
import os
import requests
import http.client
import mimetypes
from codecs import encode
import numpy as np
import argparse
import joblib
from maintain_PlatoUtils.maintain_PlatoUtils import existTag,wrapNebula2Df,existEdge,delEdge,delVertex
import joblib
import sys
import os.path as osp

csv2platoDTypeDict={
    "object":"string",
    "float64":"double",
}

def evalDataType(myDf):
    '''
    分析DataFrame的属性的数据类型
    '''
    attrTypeDict=dict(
        (colItem,csv2platoDTypeDict.get(str(myDf[colItem].dtype),colItem))
        for colItem in myDf.columns
    )
    return attrTypeDict

def uploadFolder(dataFolder,platoUrl,cookie=""):
    
    from sys import _getframe
    from pathlib import Path
    frame = _getframe(1)
    curr_file = Path(frame.f_code.co_filename)
    a=str(curr_file.parent.joinpath(dataFolder).resolve())
    print(a)
    
    if "uploadSchema.json" in os.listdir(dataFolder):
        
        with open(os.path.join(dataFolder,"uploadSchema.json").replace("\\","/"),"r",encoding="utf8") as uploadSchemaFile:
            uploadSchemaJson=json.load(uploadSchemaFile)
        
        if "vertex" in uploadSchemaJson:
            for vertexTypeItem in uploadSchemaJson["vertex"]:
                filePath=vertexTypeItem["file_path"]
                uploadFile(filePath,platoUrl=platoUrl,cookie=cookie)
            
        if "edge" in uploadSchemaJson:
            for edgeTypeItem in uploadSchemaJson["edge"]:
                filePath=edgeTypeItem["file_path"]
                uploadFile(filePath,platoUrl=platoUrl,cookie=cookie)
    
def remakeRawSchema(rawSchemaJson,csvFolder="",gClient=None):
    '''
    数据需要已上传至服务器的data/csv文件夹内才能使用
    将上传至服务器的schema调整为即将上传至图数据库的schema
    同时，data/csv内的节点信息csv文件会增加newMadeSysID列
    schema数据如下：
    {
        "gDbName":"DemoSpace",
        "vertex":[
            {
                "cover_label":True,
                "file_name":"vertexData.csv",
                "node_type":"vertexClass",
                "id_col":"vertexIDColName",
                "csv2plato_attr_map":{
                    "csvAttr":"platoAttr"
                },
                "attr_type_map":{
                    "csvAttr":"platoAttrType"
                }
            },
            ......
        ],
        "edge":[
            {
                "cover_label":True,
                "file_name":"edgeInfo.csv",
                "edge_type":"edgeClass",
                "src_type":"srcClass1",
                "tgt_type":"tgtClass2",
                "src_id":"srcIdAttr",
                "tgt_id":"tgtIdAttr",
            },
            ......
        ]
    }
    '''
    ##########################################
    # vertex.cover_label
    # escape:同名不更新√
    # plus:同名仅更新属性不删除属性√
    # cover:同名直接覆盖属性×
    # ,delete:同名删除样本x
    # ,escape:不同名不处理×
    # ,cover:不同名删除×
    #
    # edge.cover_label
    # escape:同头尾节点不更新×
    # plus:同头尾节点仅更新属性不删除属性×
    # cover:同头尾节点直接覆盖属性×
    # ,delete:同头节点名删除样本x
    # ,escape:不同头节点名不处理×
    # ,cover:不同头节点名删除×
    #########################################

    if csvFolder[-1]=="/":
        csvFolder=csvFolder[:-1]
    
    for vertexI,_ in tqdm.tqdm(enumerate(rawSchemaJson["vertex"]),desc="vertex"):
        rawSchemaJson["vertex"][vertexI]["file_path"]=csvFolder+"/"+rawSchemaJson["vertex"][vertexI]["file_name"]
        csvDfItem=pd.read_csv(rawSchemaJson["vertex"][vertexI]["file_path"])
        csvDfItem[rawSchemaJson["vertex"][vertexI]["id_col"]]=csvDfItem[rawSchemaJson["vertex"][vertexI]["id_col"]].astype(str)
        vertexType=rawSchemaJson["vertex"][vertexI]["node_type"]
        csvDfItem["newMadeSysID"]=vertexType+"_"+csvDfItem[rawSchemaJson["vertex"][vertexI]["id_col"]].astype(str)

        coverLabel=rawSchemaJson["vertex"][vertexI].get("cover_label","escape,cover")
        vertexDataProcess=coverLabel.split(",")[0]
        vertexAttrProcess=coverLabel.split(",")[1]

        # 节点数据控制
        if vertexDataProcess=="cover":
            nodeType=rawSchemaJson["vertex"][vertexI]["node_type"]
            nodeIdAttr=rawSchemaJson["vertex"][vertexI]["id_col"]
            sysVertexSearchStr="LOOKUP ON {nodeType} WHERE {nodeType}.{nodeIdAttrKey}!='不可能的名字'".format(
                                                                                                                    nodeType=nodeType,
                                                                                                                    nodeIdAttrKey=nodeIdAttr
                                                                                                                )
            # print(sysVertexSearchStr)
            sysIdList=wrapNebula2Df(gClient.execute_query(sysVertexSearchStr)).values.flatten().tolist()
            delVertex(gClient,sysIdList)
        elif vertexDataProcess=="escape":
            pass
        
        # 节点属性控制
        if vertexAttrProcess=="escape":
            nodeType=rawSchemaJson["vertex"][vertexI]["node_type"]
            nodeIdAttr=rawSchemaJson["vertex"][vertexI]["id_col"]
            csvDfItem=pd.DataFrame([row for row in json.loads(csvDfItem.to_json(orient="records")) 
                                        if existTag(nodeType, nodeIdAttr, row[rawSchemaJson["vertex"][vertexI]["id_col"]],gClient)==False])
        elif vertexAttrProcess=="plus":
            
            nodeItemCInfoDf=csvDfItem.copy(deep=True)
            
            nodeType=rawSchemaJson["vertex"][vertexI]["node_type"]
            nodeIdAttr=rawSchemaJson["vertex"][vertexI]["id_col"]
            nodeIdList=nodeItemCInfoDf[rawSchemaJson["vertex"][vertexI]["id_col"]].values.flatten().tolist()
            
            searchBatch=32
            nodeItemGInfoDfList=[]
            for nodeIdI in range(0,len(nodeIdList),searchBatch):
                queryStrList=[]
                for nodeIdIItem in range(nodeIdI,min(nodeIdI+searchBatch,len(nodeIdList))):
                    queryStrItem="LOOKUP ON {nodeType} WHERE {nodeType}.{nodeIdAttr}=='{nodeIdVal}'| FETCH PROP ON {nodeType} $-.VertexID".format(nodeType=nodeType,
                                                                                                                                                nodeIdAttr=nodeIdAttr,
                                                                                                                                                nodeIdVal=nodeIdList[nodeIdIItem])
                    queryStrList.append(queryStrItem)
                queryStr=" UNION ".join(queryStrList)
                nodeItemGInfoDf=wrapNebula2Df(gClient.execute_query(queryStr))
                nodeItemGInfoDfList.append(nodeItemGInfoDf)
            nodeItemGInfoDf=pd.concat(nodeItemGInfoDfList)
            
            # print("nodeItemCInfoDf:",nodeItemCInfoDf.columns)
            # print("nodeItemGInfoDf:",nodeItemGInfoDf.columns)
            if nodeItemGInfoDf.shape[0]>0:
                for colItem in nodeItemCInfoDf.columns:
                    if colItem in nodeItemGInfoDf:
                        idAttrKey=rawSchemaJson["vertex"][vertexI]["id_col"]
                        if colItem !=  idAttrKey:
                            changedIdList=nodeItemCInfoDf.loc[(nodeItemCInfoDf[colItem].isna())|
                                                            (nodeItemCInfoDf[colItem]=="null")|
                                                            (nodeItemCInfoDf[colItem]==""),idAttrKey].values.flatten().tolist()

                            cIdDf=nodeItemCInfoDf.loc[nodeItemCInfoDf[idAttrKey].isin(changedIdList),idAttrKey]
                            gIdDf=nodeItemGInfoDf.loc[nodeItemGInfoDf[idAttrKey].isin(changedIdList),idAttrKey]
                            
                            # print(nodeItemCInfoDf.loc[nodeItemCInfoDf[idAttrKey].isin(gIdDf),:])
                            # print(nodeItemGInfoDf.loc[nodeItemGInfoDf[idAttrKey].isin(gIdDf),:])

                            nodeItemCInfoDf.loc[nodeItemCInfoDf[idAttrKey].isin(gIdDf),colItem]=\
                                nodeItemGInfoDf.loc[nodeItemGInfoDf[idAttrKey].isin(gIdDf),colItem].values
            
            csvDfItem=nodeItemCInfoDf
        elif vertexAttrProcess=="cover":
            pass
        # print("csvDfItem:",csvDfItem.head())
        
        csvDfItem.to_csv(rawSchemaJson["vertex"][vertexI]["file_path"],index=None)
        rawSchemaJson["vertex"][vertexI]["old_id_col"]=rawSchemaJson["vertex"][vertexI]["id_col"]
        rawSchemaJson["vertex"][vertexI]["id_col"]="newMadeSysID"
        if "csv2plato_attr_map" not in rawSchemaJson["vertex"][vertexI] or len(rawSchemaJson["vertex"][vertexI]["attr_type_map"])==0: # 若没有属性映射则进行同名映射
            vertexItemDf=pd.read_csv(rawSchemaJson["vertex"][vertexI]["file_path"])
            rawSchemaJson["vertex"][vertexI]["csv2plato_attr_map"]=dict((colItem,colItem) for colItem in vertexItemDf.columns)
        if "attr_type_map" not in rawSchemaJson["vertex"][vertexI] or len(rawSchemaJson["vertex"][vertexI]["attr_type_map"])==0: # 若没有属性数据类型约束则自动生成
            vertexItemDf=pd.read_csv(rawSchemaJson["vertex"][vertexI]["file_path"])
            CPAttrTypeDict=evalDataType(vertexItemDf)
            rawSchemaJson["vertex"][vertexI]["attr_type_map"]=CPAttrTypeDict

    for edgeI,_ in tqdm.tqdm(enumerate(rawSchemaJson["edge"]),desc="edge"):
        rawSchemaJson["edge"][edgeI]["file_path"]=csvFolder+"/"+rawSchemaJson["edge"][edgeI]["file_name"]
        csvDfItem=pd.read_csv(rawSchemaJson["edge"][edgeI]["file_path"])
        srcType=rawSchemaJson["edge"][edgeI]["src_type"]
        tgtType=rawSchemaJson["edge"][edgeI]["tgt_type"]
        edgeType=rawSchemaJson["edge"][edgeI]["edge_type"]
        srcIdAttr=rawSchemaJson["edge"][edgeI]["src_id"]
        tgtIdAttr=rawSchemaJson["edge"][edgeI]["tgt_id"]
        csvDfItem[rawSchemaJson["edge"][edgeI]["src_id"]]=csvDfItem[rawSchemaJson["edge"][edgeI]["src_id"]].astype(str).apply(lambda row:srcType+"_"+row if srcType+"_" not in row else row) # 同src的newMadeSysID对应
        csvDfItem[rawSchemaJson["edge"][edgeI]["tgt_id"]]=csvDfItem[rawSchemaJson["edge"][edgeI]["tgt_id"]].astype(str).apply(lambda row:tgtType+"_"+row if tgtType+"_" not in row else row) # 同tgt的newMadeSysID对应

        coverLabel=rawSchemaJson["edge"][edgeI].get("cover_label","escape,escape,cover")
        headEdgeDataProcess,tailEdgeDataProcess,edgeAttrDataProcess=coverLabel.split(",")

        # 头节点边样本控制
        if headEdgeDataProcess=="cover":
            oldSysId=wrapNebula2Df(gClient.execute_query("LOOKUP ON {srcType} WHERE {srcType}.{srcIdAttr}!='不可能的名字'".format(
                                                                                                                                srcType=srcType,
                                                                                                                                srcIdAttr=srcIdAttr
                                                                                                                            ))).values.tolist()
            for batchI in tqdm.tqdm(range(0,len(oldSysId),1024),desc="head edges control-cover"):
                oldSysIdGroup=[str(oldSysIdItem) for oldSysIdItem in oldSysId[batchI:batchI+1024]]
                delEdge(gClient,oldSysIdGroup)
        elif headEdgeDataProcess=="escape":
            pass

        # 尾节点边样本控制
        if tailEdgeDataProcess=="cover":
            oldSysId=wrapNebula2Df(gClient.execute_query("LOOKUP ON {tgtType} WHERE {tgtType}.{tgtIdAttr}!='不可能的名字'".format(
                                                                                                                                tgtType=tgtType,
                                                                                                                                tgtIdAttr=tgtIdAttr
                                                                                                                            ))).values.tolist()
            for batchI in tqdm.tqdm(range(0,len(oldSysId),1024),desc="tail edges control-cover"):
                oldSysIdGroup=[str(oldSysIdItem) for oldSysIdItem in oldSysId[batchI:batchI+1024]]
                delEdge(gClient,oldSysIdGroup)
        elif tailEdgeDataProcess=="escape":
            pass

        # 边属性控制
        if edgeAttrDataProcess=="escape":
            stList=csvDfItem.loc[:,[srcIdAttr,tgtIdAttr]].values.tolist()
            newStList=[]
            csvDfItemList=[]
            batchSize=1024
            for stPairI in tqdm.tqdm(range(0,len(stList),batchSize),desc="edge attr control-escape"):
                srcItemType=[stPairItem[0].split("_")[0] for stPairItem in stList[stPairI:stPairI+batchSize]]
                srcItemIdValList=["_".join(stPairItem[0].split("_")[1:]) for stPairItem in stList[stPairI:stPairI+batchSize]]
                tgtItemType=[stPairItem[1].split("_")[0] for stPairItem in stList[stPairI:stPairI+batchSize]]
                tgtItemIdValList=["_".join(stPairItem[1].split("_")[1:]) for stPairItem in stList[stPairI:stPairI+batchSize]]
                for stPairII in range(len(tgtItemIdValList)):
                    srcItemIdKey=srcIdAttr
                    srcItemIdVal=srcItemIdValList[stPairII]
                    tgtItemIdKey=tgtIdAttr
                    tgtItemIdVal=tgtItemIdValList[stPairII]
                    if existEdge(srcItemType,srcItemIdKey,srcItemIdVal,tgtItemType,tgtItemIdKey,tgtItemIdVal,edgeType,gClient)==False:
                        srcKV="{}_{}".format(srcItemType,srcItemIdVal)
                        tgtKV="{}_{}".format(tgtItemIdKey,tgtItemIdVal)
                        newStList.append([srcKV,tgtKV])
                        csvDfItemRow={srcIdAttr:srcKV,tgtIdAttr:tgtKV}
                        csvDfItemList.append(csvDfItemRow)
            csvDfItem=pd.DataFrame(csvDfItemList)
        elif edgeAttrDataProcess=="plus":
            htList=json.loads(csvDfItem.to_json(orient="records"))
            newHTList=[]
            htSearchBatchSize=128
            for htI in tqdm.tqdm(range(0,len(htList),htSearchBatchSize),desc="edge attr control-plus"):
                queryStrList=[]
                for htItem in htList[htI:htI+htSearchBatchSize]:
                    htItem=htList[htI]
                    srcIdVal=htItem[srcIdAttr]
                    tgtIdVal=htItem[tgtIdAttr]
                    attrList=[keyItem for keyItem in htItem if keyItem not in [srcIdAttr,tgtIdAttr]]
                    attrGroupStr=",".join(["{edgeType}.{attrName} AS {attrName}".format(edgeType=edgeType,attrName=attrItem) for attrItem in attrList])
                    queryItemStr="LOOKUP ON {headType} WHERE {headType}.{headIdAttrKey}=='{headIdAttrVal}'|\
                                            GO FROM $-.VertexID OVER {edgeType} WHERE $$.{tailType}.{tailIdAttrKey}=='{tailIdAttrVal}'\
                                                YIELD {attrGroup}".format(
                                                    headType=srcType,
                                                    headIdAttrKey=srcIdAttr,
                                                    headIdAttrVal=srcIdVal,
                                                    edgeType=edgeType,
                                                    tailType=tgtType,
                                                    tailIdAttrKey=tgtIdAttr,
                                                    tailIdAttrVal=tgtIdVal,
                                                    attrGroup=attrGroupStr
                                                )
                    queryStrList.append(queryItemStr)
                queryStr=" UNION ".format(queryStrList)
                htItemDf=wrapNebula2Df(gClient.execute_query(queryStr))
                newHTItemList=json.loads(htItemDf.to_json(orient="records"))
                for keyItem in attrList:
                    if newHTItemList[0][keyItem] is None or np.nan(newHTItemList[0][keyItem]) or newHTItemList[0][keyItem]=="null":
                        newHTItemList[0][keyItem]=htItem[keyItem]
                newHTList+=newHTItemList
            csvDfItem=pd.DataFrame(newHTList)
        elif edgeAttrDataProcess=="cover":
            pass

        csvDfItem.to_csv(rawSchemaJson["edge"][edgeI]["file_path"],index=None)
        if "csv2plato_attr_map" not in rawSchemaJson["edge"][edgeI] or len(rawSchemaJson["edge"][edgeI]["attr_type_map"])==0:
            edgeItemDf=pd.read_csv(rawSchemaJson["edge"][edgeI]["file_path"])
            rawSchemaJson["edge"][edgeI]["csv2plato_attr_map"]=dict((colItem,colItem) for colItem in edgeItemDf.columns)
        if "attr_type_map" not in rawSchemaJson["edge"][edgeI] or len(rawSchemaJson["edge"][edgeI]["attr_type_map"])==0:
            edgeItemDf=pd.read_csv(rawSchemaJson["edge"][edgeI]["file_path"])
            CPAttrTypeDict=evalDataType(edgeItemDf)
            rawSchemaJson["edge"][edgeI]["attr_type_map"]=CPAttrTypeDict
    
    return rawSchemaJson

def buildGraphDB(gClient,uploadSchemaJson,realDbName=None,force=False):
    
    typeDefaultDict={
        "string":"null",
        "double":0.0,
        "int":0
    }
    
    if force==False:
        isexist="IF EXIST"
    else:
        isexist=""

    if realDbName is None:
        realDbName=uploadSchemaJson["gDbName"]
    
    queryList=["DROP SPACE {} {};CREATE SPACE IF NOT EXISTS {}".format(isexist,realDbName,realDbName)]
    
    for vertexInfo in uploadSchemaJson["vertex"]:
        attrNameTypeGroup=[(vertexInfo["csv2plato_attr_map"][attrKeyItem],
                            vertexInfo["attr_type_map"].get(attrKeyItem,"string"))
                           for attrKeyItem in vertexInfo["csv2plato_attr_map"]]
        attrNameTypeGroupStr=",".join([" ".join(groupItem)+" DEFAULT {}".format(typeDefaultDict[groupItem[1]]) 
                                       for groupItem in attrNameTypeGroup])
        queryItemStr="CREATE TAG IF NOT EXISTS {vertexType}({attrGroupStr})".format(
                                                                    vertexType=vertexInfo["node_type"],
                                                                    attrGroupStr=attrNameTypeGroupStr
                                                                    )
        indexQueryItemStr="CREATE TAG INDEX IF NOT EXISTS \
                            {tagTypeLower}_{tagIndexLower}_index ON {tagType}({vertexIdAttr});\
                            REBUIL TAG INDEX {tagTypeLower}_{tagIndexLower}_index OFFLINE".format(
                                                                        tagType=vertexInfo["node_type"],
                                                                        vertexIdAttr=vertexInfo["id_col"],
                                                                        tagTypeLower=vertexInfo["node_type"].lower(),
                                                                        tagIndexLower=vertexInfo["id_col"].lower()
                                                                    )
        queryList.append(queryItemStr)
        queryList.append(indexQueryItemStr)
        
    for edgeInfo in uploadSchemaJson["edge"]:
        attrNameTypeGroup=[(edgeInfo["csv2plato_attr_map"][attrKeyItem],
                            edgeInfo["attr_type_map"].get(attrKeyItem,"string"))
                           for attrKeyItem in edgeInfo["csv2plato_attr_map"]]
        attrNameTypeGroupStr=",".join([",".join(groupItem)+" DEFAULT {}".format(typeDefaultDict[groupItem[1]])
                                       for groupItem in attrNameTypeGroup])
        queryItemStr="CREATE EDGE IF NOT EXISTS {edgeType}({attrGroupStr})".format(
                                                                    edgeType=edgeInfo["edge_type"],
                                                                    attrGroupStr=attrNameTypeGroupStr
                                                                    )
        queryList.append(queryItemStr)
    
    queryStr=";".join(queryList)
    gClient.execute_query(queryStr)

def uploadFile(filepath,platoUrl="http://9.135.95.249:7001",cookie=""):

    url = platoUrl+"/api/files/upload"

    newFilePath=filepath[:filepath.index(".csv")]+"_copy.csv"
    pd.read_csv(filepath).to_csv(newFilePath,header=None,index=None)
    dataList = []
    boundary = 'wL36Yn8afVp8Ag7AmP8qZ0SA4n1v9T'
    dataList.append(encode('--' + boundary))
    dataList.append(encode('Content-Disposition: form-data; name=filename; filename={0}'.format(newFilePath)))

    fileType = mimetypes.guess_type(newFilePath)[0] or 'application/octet-stream'
    dataList.append(encode('Content-Type: {}'.format(fileType)))
    dataList.append(encode(''))

    with open(newFilePath, 'rb') as f:
        dataList.append(f.read())
    dataList.append(encode('--' + boundary))
    dataList.append(encode('Content-Disposition: form-data; name=Content-Disposition;'))

    dataList.append(encode('Content-Type: {}'.format('text/plain')))
    dataList.append(encode(''))

    dataList.append(encode("form-data"))
    dataList.append(encode('--' + boundary))
    dataList.append(encode('Content-Disposition: form-data; name=name;'))
    body = b'\r\n'.join(dataList)

    headers = {
        'Proxy-Connection': 'keep-alive',
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.107 Safari/537.36 Edg/92.0.902.62',
        'X-Requested-With': 'XMLHttpRequest',
        'Accept': '*/*',
        'Origin': platoUrl,
        'Referer': platoUrl+'/import',
        'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6',
        'Cookie': cookie,
        'Content-type': 'multipart/form-data; boundary={}'.format(boundary)
    }

    response = requests.request("POST", url, headers=headers, data=body)

    print(response.text)

if __name__=="__main__":
    
    dataFolder="csv2platodb/attr2Vertex_1629250000"
    platoUrl="http://9.135.95.249:7001"
    # platoUrl="http://10.99.218.40:8081"
    
    uploadFolder(dataFolder,platoUrl)
    
    print("finished")
                



from string import punctuation
import re

def preprocessSentence(mySentence,methods="lnp"):
    '''
    预处理句子，并分词
    methods:
    l-lower:小写字母
    n-num:替换数字为num
    p-punctuation:删除标点符号
    '''
    mySentence=mySentence.strip()
    puncStr=punctuation+"「」（）？！?!<>《》、“”。，：♫︰【】—；"
    if "l" in methods:
        mySentence=mySentence.lower()
    mySentence=re.sub("([a-zA-Z]+)",r" \1 ",mySentence)
    
    if "n" in methods:
        mySentence=re.sub("([0-9]+)"," num ",mySentence)
        
    if "p" in methods:
        mySentence=re.sub("["+puncStr+"]+","",mySentence)
    
    mySentence=" ".join([charItem for charItem in mySentence.split(" ") if len(charItem)>0])
    mySentenceList=[]
    for wordItem in mySentence.split(" "):
        if len(re.findall("[\u4e00-\u9fa5]",wordItem))==len(wordItem):
            for charItem in wordItem:
                mySentenceList.append(charItem)
        else:
            mySentenceList.append(wordItem)
    tmpSentence=" ".join(mySentenceList)
    return tmpSentence               

def getSEOfEntity_forCH(entityItem,mySentenceList):
    '''
    获取中文实体entityItem列表头尾id
    '''
    seList=[]
    for tokenI,tokenItem in enumerate(mySentenceList):
        if tokenItem==entityItem[0]:
            plusTokenI=0
            startI=tokenI+plusTokenI
            while plusTokenI<len(entityItem) and tokenI+plusTokenI<len(mySentenceList) and mySentenceList[tokenI+plusTokenI]==entityItem[plusTokenI]:
                plusTokenI+=1
            if plusTokenI==len(entityItem):
                endI=tokenI+plusTokenI
                seList.append([startI,endI])
    return seList

def getSEOfEntity_forEN(entityItem,mySentenceList):
    '''
    获取英文实体entityItem列表头尾id
    '''
    entityItemList=entityItem.split(" ")
    mySentenceList=mySentenceList
    seList=[]
    for tokenI,tokenItem in enumerate(mySentenceList):
        if tokenItem==entityItem[0]:
            plusTokenI=0
            startI=tokenI+plusTokenI
            while plusTokenI<len(entityItemList) and tokenI+plusTokenI<len(mySentenceList) and mySentenceList[tokenI+plusTokenI]==entityItemList[plusTokenI]:
                plusTokenI+=1
            if plusTokenI==len(entityItemList):
                endI=tokenI+plusTokenI
                seList.append([startI,endI])
    return seList

def tagSentence(mySentence,entityList):
    '''
    为句子mySentence标注entityList中的实体
    '''
    mySentenceList=preprocessSentence(mySentence).split(" ")
    
    BIOList=["O" for tagItem in mySentenceList]
    
    mySentence_forCH="".join(mySentenceList)
    mySentence_forEN=" ".join(mySentenceList)
    seList=[]
    for entityI,entityItem in enumerate(entityList):
        entityItem=entityItem.lower()
        if len(re.findall("[\u4e00-\u9fa5]",entityItem))>0:
            if entityItem in mySentence_forCH:
                seItem=getSEOfEntity_forCH(entityItem,mySentenceList)
                seList+=seItem
        elif entityItem in mySentenceList:
            if entityItem in mySentence_forEN:
                seItem=getSEOfEntity_forEN(entityItem,mySentenceList)
                seList+=seItem
    
    for seItem in seList:
        startI,endI=seItem
        BIOList[startI]="B"
        for charI in range(startI+1,endI):
            BIOList[charI]="I"
    
    return " ".join(BIOList)

if __name__=="__main__":
    mySent="实验设计【启航计划】\n"
    
    processedSent=preprocessSentence(mySent)
    print(processedSent.split(" "))
    
    tagList=tagSentence(mySent,["广告","api","投放"])
    print(tagList.split(" "))


from nebula.graph import ttypes,GraphService
from nebula.ConnectionPool import ConnectionPool
from nebula.Client import GraphClient
import pandas as pd
import numpy as np
import tqdm
import os
import time
import json
import requests
import gc
import random
from string import punctuation
import argparse
from string import punctuation
import re
import shutil

def wrapNebula2Df(nebulaObj):
    '''将platoDB查询到的对象转为df'''
    # print(nebulaObj.column_names)
    start=time.time()
    if nebulaObj.column_names is not None:
        columnList = [colItem.decode("utf8") for colItem in nebulaObj.column_names]
    else:
        return pd.DataFrame([])
    
    dataList = []
    if nebulaObj.rows is not None:
        gc.disable()
        for rowItem in nebulaObj.rows:
            rowList = []
            for colItem in rowItem.columns:
                if type(colItem.value) == bytes:
                    rowList.append(colItem.value.decode("utf8"))
                else:
                    rowList.append(colItem.value)
            dataList.append(rowList.copy())
        gc.enable()
    else:
        return pd.DataFrame([])
    
    return pd.DataFrame(dataList, columns=columnList).drop_duplicates()

def pdPlatoTypeSame(pdSeries,gType):
    '''pd.DataFrame的series的数据类型是否和gType一致'''
    tmpPdSeries=pdSeries.dropna()
    if gType=="string":
        if tmpPdSeries.dtype==object:
            return True
    elif gType=="int":
        if tmpPdSeries.dtype==np.int64:
            return True
    elif gType=="double":
        if tmpPdSeries.dtype==np.float64:
            return True
    return False

def delEdge(gClient,sysIdList,edgeType="*"):
    '''删除与源节点sysIdList关联的边edgeType'''
    if edgeType=="*":
        relDf=wrapNebula2Df(gClient.execute_query("SHOW EDGES"))["Name"]
        relList=relDf.values.flatten().tolist()
    else:
        relList=[edgeType]
    for relItem in tqdm.tqdm(relList,desc="del edges"):
        for srcSysIdItem in sysIdList:
            searchRelStr="GO FROM {srcSysId} OVER {edgeName} BIDIRECT YIELD {edgeName}._dst AS tgtSysId".format(
                                                                                                                srcSysId=srcSysIdItem,
                                                                                                                edgeName=relItem)
            print(searchRelStr)
            relTailSysIdDf=wrapNebula2Df(gClient.execute_query(searchRelStr))
            if relTailSysIdDf.shape[0]>0:
                relTailSysIdList=relTailSysIdDf["tgtSysId"].values.flatten().tolist()
                delOrderGroupStr=",".join(["{}->{}".format(srcSysIdItem,tailSysIdItem) for tailSysIdItem in relTailSysIdList])
                delReverseGroupStr=",".join(["{}->{}".format(tailSysIdItem,srcSysIdItem) for tailSysIdItem in relTailSysIdList])
                delGroupStr=",".join([delOrderGroupStr,delReverseGroupStr])
                delStr="DELETE EDGE {} {}".format(relItem,delGroupStr)
                print(delStr)
                gClient.execute_query(delStr)

def delVertex(gClient,sysIdList,delRel=True):
    '''（关联）删除节点'''
    delReq={"data":[],"error_code":0}
    if delRel==True:
        relDf=wrapNebula2Df(gClient.execute_query("SHOW EDGES"))
        if relDf.shape[0]>0:
            relList=relDf["Name"].values.flatten().tolist()
            for relItem in tqdm.tqdm(relList,desc="del edges with vertexes"):
                for srcSysIdItem in tqdm.tqdm(sysIdList,desc="del srcs from edges with vertexes"):
                    relTailStr="GO FROM {srcSysId} OVER {edgeName} BIDIRECT YIELD {edgeName}._dst AS tgtSysId".format(
                        srcSysId=srcSysIdItem,
                        edgeName=relItem)
                    # print(relTailStr)
                    relTailSysIdDf=wrapNebula2Df(gClient.execute_query(relTailStr))
                    if relTailSysIdDf.shape[0]>0:
                        relTailSysIdList=relTailSysIdDf["tgtSysId"].values.flatten().tolist()
                        delOrderGroupStr=",".join(["{}->{}".format(srcSysIdItem,tailSysIdItem) for tailSysIdItem in relTailSysIdList])
                        delReverseGroupStr=",".join(["{}->{}".format(tailSysIdItem,srcSysIdItem) for tailSysIdItem in relTailSysIdList])
                        delGroupStr=",".join([delOrderGroupStr,delReverseGroupStr])
                        delStr="DELETE EDGE {} {}".format(relItem,delGroupStr)
                        # print(delStr)
                        gClient.execute_query(delStr)
    # print(sysIdList)
    for batchI in tqdm.tqdm(range(0,len(sysIdList),50),desc="del vertexes"): 
        delVerGroupStr=",".join([str(sysIdItem) for sysIdItem in sysIdList[batchI:batchI+50]])
        delStr="DELETE VERTEX {}".format(delVerGroupStr)
        # print(delStr)
        delReq=gClient.execute_query(delStr)
    return delReq
                
def existTag(nodeType,nodeIdAttr,nodeName,gClient):
    '''查看nodeType的nodeIdAttr为nodeName的节点是否在gClient中（gClient提前设定好图数据库）'''
    passTag=0
    while passTag<5:
        try:
            searchTagDf=wrapNebula2Df(gClient.execute_query("LOOKUP ON {nodeType} WHERE {nodeType}.{nodeIdAttr}=='{nodeName}'|LIMIT 1".format(
                nodeType=nodeType,
                nodeIdAttr=nodeIdAttr,
                nodeName=nodeName
            )))
            if searchTagDf.shape[0]>0:
                return True
            else:
                passTag+=1
        except:
            passTag+=1
    return False

def transferBetweenPlato(gHost,gPort,gUser,gPassword,
                         srcGdbName,tgtGdbName,tgtGAPIUrl,edgeTypeList=[],
                         srcVertexKeynameDict={"srcNodeType":"srcNodeIdAttr"},csv2platoDTypeDict={"srcNodeIdAttr":"string"},
                         batchSize=64,projectName="",platoAPIIP="",platoAPIPort=8083):
    
    srcConnection_pool = ConnectionPool(gHost, gPort,network_timeout=300000)
    srcClient = GraphClient(srcConnection_pool)
    srcClient.authenticate(gUser, gPassword)
    srcClient.execute_query("use {}".format(srcGdbName))

    tgtConnection_pool = ConnectionPool(gHost, gPort,network_timeout=300000)
    tgtClient = GraphClient(tgtConnection_pool)
    tgtClient.authenticate(gUser, gPassword)
    tgtClient.execute_query("use {}".format(tgtGdbName))

    # 1.构建data的项目

    # 获取schema
    srcVertexTypeDf=wrapNebula2Df(srcClient.execute_query("SHOW TAGS"))
    srcVertexTypeAttrSetDict={}
    for srcVertexTypeItem in srcVertexTypeDf["Name"].values.tolist():
        tagTypeListStr="DESCRIBE TAG {}".format(srcVertexTypeItem)
        srcVertexInfoDf=wrapNebula2Df(srcClient.execute_query(tagTypeListStr))
        srcVertexTypeAttrSetDict[srcVertexTypeItem]=dict(srcVertexInfoDf.loc[:,["Field","Type"]].values.tolist())
    
    srcEdgeTypeDf=wrapNebula2Df(srcClient.execute_query("SHOW EDGES"))
    srcEdgeTypeAttrSetDict={}
    for srcEdgeTypeItem in srcEdgeTypeDf["Name"].values.tolist():
        if len(edgeTypeList)==0 or srcEdgeTypeItem in edgeTypeList:
            edgeTypeListStr="DESCRIBE EDGE {}".format(srcEdgeTypeItem)
            srcEdgeInfoDf=wrapNebula2Df(srcClient.execute_query(edgeTypeListStr))
            srcEdgeTypeAttrSetDict[srcEdgeTypeItem]=dict(srcEdgeInfoDf.loc[:,["Field","Type"]].values.tolist())
    
    # 构建schema
    for srcVertexTypeAttrSetItem in srcVertexTypeAttrSetDict:
        srcVertexTypeSet=srcVertexTypeAttrSetDict[srcVertexTypeAttrSetItem]
        tagAttrStr=",".join(["{} {}".format(srcVertexTypeItem,srcVertexTypeSet[srcVertexTypeItem] if srcVertexTypeSet[srcVertexTypeItem] not in ["int","double"] else srcVertexTypeSet[srcVertexTypeItem]+" DEFAULT 0") for srcVertexTypeItem in srcVertexTypeSet])
        buildTagSchemaStr="CREATE TAG IF NOT EXISTS {}({}) ".format(srcVertexTypeAttrSetItem,tagAttrStr)
        tgtClient.execute_query(buildTagSchemaStr)

    
    for srcEdgeTypeAttrSetItem in srcEdgeTypeAttrSetDict:
        srcEdgeTypeSet=srcEdgeTypeAttrSetDict[srcEdgeTypeAttrSetItem]
        tagAttrStr=",".join(["{} {}".format(srcEdgeTypeItem,srcEdgeTypeSet[srcEdgeTypeItem]) for srcEdgeTypeItem in srcEdgeTypeSet])
        buildEdgeSchemaStr="CREATE EDGE IF NOT EXISTS {}({}) ".format(srcEdgeTypeAttrSetItem,tagAttrStr)
        tgtClient.execute_query(buildEdgeSchemaStr)

    # 构建index
    for vertexTypeItem in srcVertexKeynameDict:
        tagIndexName="{}_{}_index".format(vertexTypeItem.lower(),srcVertexKeynameDict[vertexTypeItem].lower())
        tgtClient.execute_query("CREATE TAG INDEX IF NOT EXISTS {} ON {}({})".format(tagIndexName,vertexTypeItem,srcVertexKeynameDict[vertexTypeItem]))
        tgtClient.execute_query("REBUILD TAG INDEX {} OFFLINE".format(tagIndexName))

    # 获取nebula graph导入形式的数据
    if len(projectName)==0:
        projectName="tmpProject_{}".format(int(time.time()*1000))
    if "csv2plato" not in os.listdir("."):
        os.mkdir("csv2plato")
    if projectName not in os.listdir("csv2plato"):
        os.mkdir("csv2plato/"+projectName)
    rawSchemaJson={
        "gDbName":tgtGdbName,
        "coverOldData":True, 
        "vertex":[],
        "edge":[]
    }
    vertexRecordList=[]
    edgeRecordList=[]
    for srcVertexTypeItem in tqdm.tqdm(srcVertexKeynameDict):
        batchI=0
        while True:
            vertexSysIdDf=wrapNebula2Df(srcClient.execute_query("LOOKUP ON {vertexType} WHERE {vertexType}.{attrKeyname}!='不可能的名字'|LIMIT {batchI},{batchSize}".format(
                                                                                                                                                vertexType=srcVertexTypeItem,
                                                                                                                                                attrKeyname=srcVertexKeynameDict[srcVertexTypeItem],
                                                                                                                                                batchI=batchI,
                                                                                                                                                batchSize=batchSize
            )))
            if vertexSysIdDf.shape[0]==0:
                break
            vertexSysIdList=vertexSysIdDf["VertexID"].values.tolist()
            vertexSysIdList=[str(vertexSysIdItem) for vertexSysIdItem in vertexSysIdList]

            vertexInfoDf=wrapNebula2Df(srcClient.execute_query("FETCH PROP ON {} {}".format(srcVertexTypeItem,",".join(vertexSysIdList))))
            while vertexInfoDf.shape[0]==0:
                vertexInfoDf=wrapNebula2Df(srcClient.execute_query("FETCH PROP ON {} {}".format(srcVertexTypeItem,",".join(vertexSysIdList))))
                print("line wrong,check!")
            columnList=list(vertexInfoDf.columns)
            columnRenameDict=dict((colItem,colItem.split(".")[1]) for colItem in columnList if "." in colItem)
            vertexInfoDf.rename(columnRenameDict,axis=1,inplace=True)
            vertexInfoDf.drop("VertexID",axis=1,inplace=True)
            vertexInfoDf["{}SysId".format(srcVertexTypeItem)]=vertexInfoDf["{}".format(srcVertexKeynameDict[srcVertexTypeItem])].apply(lambda row:"{}".format(srcVertexTypeItem)+"_"+row)
            if batchI==0:
                vertexInfoDf.to_csv("csv2plato/{}/{}Node-fornew.csv".format(projectName,srcVertexTypeItem),index=None)
            else:
                vertexInfoDf.to_csv("csv2plato/{}/{}Node-fornew.csv".format(projectName,srcVertexTypeItem),header=None,index=None,mode="a")
            csv2platoAttrMapDict=dict((colItem,colItem) for colItem in vertexInfoDf.columns)
            csvAttrTypeDict=dict((colItem,csv2platoDTypeDict[vertexInfoDf[colItem].dtype.name]) for colItem in vertexInfoDf.columns)

            if srcVertexTypeItem not in vertexRecordList:
                rawSchemaJson["vertex"].append({
                    "file_name":"{}Node-fornew.csv".format(srcVertexTypeItem),
                    "node_type":srcVertexTypeItem,
                    "id_col":srcVertexKeynameDict[srcVertexTypeItem],
                    "csv2plato_attr_map":csv2platoAttrMapDict,
                    "attr_type_map":csvAttrTypeDict
                })
                vertexRecordList.append(srcVertexTypeItem)

            for srcEdgeTypeItem in srcEdgeTypeAttrSetDict:
                for tgtVertexTypeItem in srcVertexKeynameDict:
                    attrListStr=",".join(["{}.{}".format(srcEdgeTypeItem,edgeItem) for edgeItem in srcEdgeTypeAttrSetDict[srcEdgeTypeItem]])
                    if len(attrListStr)==0:
                        attrListStr=""
                    else:
                        attrListStr=","+attrListStr
                    goDf=wrapNebula2Df(srcClient.execute_query("GO FROM {headSysId} OVER {edge} YIELD $^.{headType}.{headKeyname} AS headId,$$.{tailType}.{tailKeyname} AS tailId{attrList}".format(
                        headSysId=",".join(vertexSysIdList),
                        edge=srcEdgeTypeItem,
                        headType=srcVertexTypeItem,
                        headKeyname=srcVertexKeynameDict[srcVertexTypeItem],
                        tailType=tgtVertexTypeItem,
                        tailKeyname=srcVertexKeynameDict[tgtVertexTypeItem],
                        attrList=attrListStr
                    )))
                    goDf.replace("",np.nan,inplace=True)
                    goDf.dropna(inplace=True)
                    if goDf.shape[0]>0:

                        goDf["headId"]=goDf["headId"].apply(lambda row:"{}_".format(srcVertexTypeItem)+row)
                        goDf["tailId"]=goDf["tailId"].apply(lambda row:"{}_".format(tgtVertexTypeItem)+row)
                        columnRenameDict=dict((colItem,colItem.split(".")[1]) for colItem in goDf.columns if "." in colItem)
                        goDf.rename(columnRenameDict,axis=1,inplace=True)

                        if "{}Rel-fornew.csv".format(srcEdgeTypeItem) not in os.listdir("csv2plato/{}/".format(projectName)):
                            goDf.to_csv("csv2plato/{}/{}Rel-fornew.csv".format(projectName,srcEdgeTypeItem),index=None)
                        else:
                            goDf.to_csv("csv2plato/{}/{}Rel-fornew.csv".format(projectName,srcEdgeTypeItem),index=None,header=None,mode="a")
                        
                        csv2platoAttrMapDict=dict((colItem,colItem) for colItem in goDf.columns if colItem not in ["headId","tailId"])
                        csvAttrTypeDict=dict((colItem,csv2platoDTypeDict[goDf[colItem].dtype.name]) for colItem in goDf.columns if colItem not in ["headId","tailId"])
                        
                        if srcEdgeTypeItem not in edgeRecordList:
                            rawSchemaJson["edge"].append({
                                "file_name":"{}Rel-fornew.csv".format(srcEdgeTypeItem),
                                "edge_type":srcEdgeTypeItem,
                                "src_type":srcVertexTypeItem,
                                "tgt_type":tgtVertexTypeItem,
                                "src_id":"headId",
                                "tgt_id":"tailId",
                                "csv2plato_attr_map":csv2platoAttrMapDict,
                                "attr_type_map":csvAttrTypeDict
                            })
                            edgeRecordList.append(srcEdgeTypeItem)

            batchI+=batchSize

    with open("csv2plato/{}/rawSchema.json".format(projectName),"w+",encoding="utf8") as rawSchemaJsonFile:
        json.dump(rawSchemaJson,rawSchemaJsonFile)

    fileList=[]
    for fileItem in tqdm.tqdm(os.listdir("csv2plato/"+projectName)):
        if fileItem.split(".")[1]=="csv":
            fileList.append(("csv",(fileItem,open("csv2plato/"+projectName+"/"+fileItem,'rb'),"test/csv")))
        if fileItem.split(".")[1]=="json":
            fileList.append(("json",(fileItem,open("csv2plato/"+projectName+"/"+fileItem,'rb'),"application/json")))

    tgtGAPIUrl=tgtGAPIUrl
    fileServerUrl="{}/csv2platodb/upload".format(tgtGAPIUrl)
    response = requests.request("POST", fileServerUrl, files=fileList)
    
    shutil.rmtree("csv2plato/"+projectName)
        
    print(response.text)
    return response

def existEdge(headType,headIdAttr,headIdVal,tailType,tailIdAttr,tailIdVal,edgeType,gClient):
    '''查看头节点为headIdVal，尾节点为tailIdVal的edgeType边是否存在（gClient提前设定好图数据库）'''
    searchEdgeDf=wrapNebula2Df(gClient.execute_query("LOOKUP ON {headType} WHERE {headType}.{headIdAttr}=='{headIdVal}'|\
                                                        GO FROM $-.VertexID OVER {edgeType} WHERE  $$.{tailType}.{tailIdAttr}=='{tailIdVal}'|LIMIT 1\
                                                            YIELD $$.{tailType}.{tailIdAttr}".format(
                                                                                                headType=headType,
                                                                                                headIdAttr=headIdAttr,
                                                                                                headIdVal=headIdVal,
                                                                                                edgeType=edgeType,
                                                                                                tailType=tailType,
                                                                                                tailIdAttr=tailIdAttr,
                                                                                                tailIdVal=tailIdVal
                                                                                            )))
    if searchEdgeDf.shape[0]>0:
        return True
    return False

def nlpPreprocessAttr(gClient,nodeType,nodeIdAttrName,nodeAttrName,tabuStr=""):
    '''图数据库属性的文本预处理'''
    tabuStr="\\/'"+tabuStr+punctuation
    nodeSysIdDf=wrapNebula2Df(gClient.execute_query("LOOKUP ON {nodeType} WHERE {nodeType}.{nodeIdAttr}!='不可能的名字'\
                                                        YIELD {nodeType}.{nodeAttr} AS tgtAttr".format(nodeType=nodeType,
                                                                                            nodeIdAttrName=nodeIdAttrName,
                                                                                            nodeAttrName=nodeAttrName)))
    if nodeSysIdDf.shape[0]>0:
        nodeSysIdDf["tgtAttr"]=nodeSysIdDf["tgtAttr"].apply(lambda row:re.sub("["+tabuStr+"]","",row))
        nodeSysIdAttrList=nodeSysIdDf.loc[:,["VertexID","tgtAttr"]].values.tolist()
        for nodeSysId,nodeAttrVal in nodeSysIdAttrList:
            gClient.execute_query("UPDATE VERTEX {nodeSysId} SET {nodeType}.{nodeAttr}='{nodeVal}'".format(
                                                                                                            nodeSysId=nodeSysId,
                                                                                                            nodeType=nodeType,
                                                                                                            nodeAttr=nodeAttrName,
                                                                                                            nodeVal=nodeAttrVal
                                                                                                        ))
        
        
    

def addTokenForEntities(gClient,gDBName,nodeIdAttrList,tabuList=[],projectName="",email=""):
    '''构建实体的token实体'''
    if "csv2platodb" not in os.listdir("."):
        os.mkdir("csv2platodb")
        
    if len(projectName)==0:
        projectName="tmp_{}".format(int(time.time()*1000))
        os.mkdir("csv2platodb/{}".format(projectName))
    
    if len(tabuList)==0:
        tabuList=list(punctuation)+["“","”","‘","’","，","。"]
    
    # 构建节点和边的数据
    schemaVertex=[]
    schemaEdge=[]
    for nodeItdPairItem in nodeIdAttrList:
        
        nodeType=nodeItdPairItem.split(".")[0]
        nodeIdAttr=nodeItdPairItem.split(".")[1]
        
        tgtNameDf=wrapNebula2Df(gClient.execute_query("LOOKUP ON {nodeType} WHERE {nodeType}.{nodeIdAttr}!='不可能的名字' \
                                                        YIELD {nodeType}.{nodeIdAttr} AS tgtName".format(nodeType=nodeType,
                                                                                                          nodeIdAttr=nodeIdAttr)))
        srcTokenPairList=[]
        tokenInfoList=[]
        for tgtNameItem in tqdm.tqdm(tgtNameDf["tgtName"].values.flatten().tolist(),desc=nodeType):
            tgtNameItem=tgtNameItem.lower()
            for tgtNameTokenItem in tgtNameItem:
                if tgtNameTokenItem not in tabuList:
                    tokenInfoList.append({"Name":tgtNameTokenItem})
                # print({"headEntity":tgtNameItem,"tailEntity":tgtNameTokenItem})
                srcTokenPairList.append({"headEntity":tgtNameItem,"tailEntity":tgtNameTokenItem})
        srcTokenPairDf=pd.DataFrame(srcTokenPairList)
        tokenInfoDf=pd.DataFrame(tokenInfoList)
            
        tokenInfoDf.to_csv("./csv2platodb/{}/tokenInfo_{}.csv".format(projectName,nodeType),index=None)
        srcTokenPairDf.to_csv("./csv2platodb/{}/consistOfRel_{}.csv".format(projectName,nodeType),index=None)
        
        schemaVertex.append({ 
                                "file_name":"tokenInfo_{}.csv".format(nodeType),
                                "node_type":"Token",
                                "id_col":"Name", 
                                "csv2plato_attr_map":{ 
                                    "Name":"Name"
                                },
                                "attr_type_map":{ 
                                    "Name":"string"
                                }
                            })
        schemaEdge.append({
                                "file_name":"consistOfRel_{}.csv".format(nodeType),
                                "edge_type":"consistOf",
                                "src_type":nodeType,
                                "tgt_type":"Token",
                                "src_id":"headEntity",
                                "tgt_id":"tailEntity",
                                "csv2plato_attr_map":{},
                                "attr_type_map":{}
                            })
    # 构建数据schema
    VESchema={
        "gDbName":gDBName,
        "vertex":schemaVertex,
        "edge":schemaEdge,
        "email":email
    }
    if len(email)==0:
        VESchema.pop("email")
    
    with open("./csv2platodb/{}/rawSchema.json".format(projectName,nodeType),"w+",encoding="utf8") as VESchemaFile:
        json.dump(VESchema,VESchemaFile)
    
    print("本地文件生成")
    return VESchema

if __name__=="__main__":
    
    srcGHost=""
    srcGPort=13708
    srcGUser="root"
    srcGPassword="nebula"
    srcGdbName="company_product_field_musklin"
    
    srcConnection_pool = ConnectionPool(srcGHost, srcGPort,network_timeout=300000)
    srcClient = GraphClient(srcConnection_pool)
    srcClient.authenticate(srcGUser, srcGPassword)
    srcClient.execute_query("use {}".format(srcGdbName))
    
    # start=time.time()
    # adf=wrapNebula2Df(srcClient.execute_query("LOOKUP ON Token WHERE Token.TokenName=='深'|GO FROM $-.VertexID OVER consistOf REVERSELY YIELD consistOf._dst AS tailSysId,$$.Company.CompanyName AS CompanyName,consistOf.weight AS weight|    ORDER BY $-.weight DESC|LIMIT 50 UNION \
    #                                             LOOKUP ON Token WHERE Token.TokenName=='圳'|GO FROM $-.VertexID OVER consistOf REVERSELY YIELD consistOf._dst AS tailSysId,$$.Company.CompanyName AS CompanyName,consistOf.weight AS weight|    ORDER BY $-.weight DESC|LIMIT 50 UNION \
    #                                             LOOKUP ON Token WHERE Token.TokenName=='市'|GO FROM $-.VertexID OVER consistOf REVERSELY YIELD consistOf._dst AS tailSysId,$$.Company.CompanyName AS CompanyName,consistOf.weight AS weight|    ORDER BY $-.weight DESC|LIMIT 50 UNION \
    #                                             LOOKUP ON Token WHERE Token.TokenName=='上'|GO FROM $-.VertexID OVER consistOf REVERSELY YIELD consistOf._dst AS tailSysId,$$.Company.CompanyName AS CompanyName,consistOf.weight AS weight|    ORDER BY $-.weight DESC|LIMIT 50 UNION \
    #                                             LOOKUP ON Token WHERE Token.TokenName=='元'|GO FROM $-.VertexID OVER consistOf REVERSELY YIELD consistOf._dst AS tailSysId,$$.Company.CompanyName AS CompanyName,consistOf.weight AS weight|    ORDER BY $-.weight DESC|LIMIT 50 UNION \
    #                                             LOOKUP ON Token WHERE Token.TokenName=='资'|GO FROM $-.VertexID OVER consistOf REVERSELY YIELD consistOf._dst AS tailSysId,$$.Company.CompanyName AS CompanyName,consistOf.weight AS weight|    ORDER BY $-.weight DESC|LIMIT 50 UNION \
    #                                             LOOKUP ON Token WHERE Token.TokenName=='本'|GO FROM $-.VertexID OVER consistOf REVERSELY YIELD consistOf._dst AS tailSysId,$$.Company.CompanyName AS CompanyName,consistOf.weight AS weight|    ORDER BY $-.weight DESC|LIMIT 50 UNION \
    #                                             LOOKUP ON Token WHERE Token.TokenName=='管'|GO FROM $-.VertexID OVER consistOf REVERSELY YIELD consistOf._dst AS tailSysId,$$.Company.CompanyName AS CompanyName,consistOf.weight AS weight|    ORDER BY $-.weight DESC|LIMIT 50 UNION \
    #                                             LOOKUP ON Token WHERE Token.TokenName=='理'|GO FROM $-.VertexID OVER consistOf REVERSELY YIELD consistOf._dst AS tailSysId,$$.Company.CompanyName AS CompanyName,consistOf.weight AS weight|    ORDER BY $-.weight DESC|LIMIT 50 UNION \
    #                                             LOOKUP ON Token WHERE Token.TokenName=='有'|GO FROM $-.VertexID OVER consistOf REVERSELY YIELD consistOf._dst AS tailSysId,$$.Company.CompanyName AS CompanyName,consistOf.weight AS weight|    ORDER BY $-.weight DESC|LIMIT 50 UNION \
    #                                             LOOKUP ON Token WHERE Token.TokenName=='限'|GO FROM $-.VertexID OVER consistOf REVERSELY YIELD consistOf._dst AS tailSysId,$$.Company.CompanyName AS CompanyName,consistOf.weight AS weight|    ORDER BY $-.weight DESC|LIMIT 50"))
    # end=time.time()
    # print(end-start)
    
    # print(adf.head())
    
    # srcVertexKeynameDict={
    #     "Company":"CompanyName",
    #     "Field":"FieldName"
    # }
    # edgeTypeList=["belongTo"]

    # tgtGHost=""
    # tgtGPort=13708
    # tgtGAPIUrl="http://{}:8083"
    # tgtGUser="root"
    # tgtGPassword="nebula"
    # tgtGdbName="company_product_field_musklin"
    
    # projectName="transform_some_data"

    # batchSize=1024
    # myDfList=[]

    # csv2platoDTypeDict={
    #     "object":"string",
    #     "float64":"double",
    #     "int64":"int",
    # }
    
    # transferBetweenPlato(srcGHost,srcGPort,srcGUser,srcGPassword,srcGdbName,
    #                      tgtGHost,tgtGPort,tgtGUser,tgtGPassword,tgtGdbName,tgtGAPIUrl,
    #                      edgeTypeList=[],
    #                      srcVertexKeynameDict=srcVertexKeynameDict,csv2platoDTypeDict=csv2platoDTypeDict)
    
    # pass



import setuptools
from setuptools import setup

NAME = "maintain_PlatoUtils"
VERSION = "0.1.3.2"
PY_MODULES = ["maintain_PlatoUtils"]

with open("README.md", "r", encoding="utf8") as fh:
    long_description = fh.read()

setup(
    name=NAME,
    version=VERSION,
    py_modules=PY_MODULES,
    packages=setuptools.find_packages(),
    url='https://github.com/Timaos123/maintian_PlatoUtils.git',
    license='MIT',
    author='Timaos',
    author_email='201436009@uibe.edu.cn',
    description='运营PlatoDB的工具',
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=['numpy',
                      "nebula-python",
                      "pandas",
                      "gensim",
                      "tensorflow",
                      "stellargraph",
                      "networkx",
                      "chardet"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.0',
)



import requests
import json


def importData(gUrl,taskId="12",cookie=""):

    url="{}/api/import/import".format(gUrl)

    payload = json.dumps({
        "taskId": taskId
    })
    headers = {
        'Proxy-Connection': 'keep-alive',
        'Accept': 'application/json, text/plain, */*',
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.107 Safari/537.36 Edg/92.0.902.62',
        'Content-Type': 'application/json',
        'Origin': 'http://9.135.95.249:7001',
        'Referer': 'http://9.135.95.249:7001/import',
        'Accept-Language': 'zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6',
        'Cookie': cookie
    }

    response = requests.request("POST", url, headers=headers, data=payload)
    print(response.text)

    return response.json()


if __name__=="__main__":
    
    # test
    ghost="9.135.95.249"
    gport=13708
    guser="root"
    gpassword="nebula"
    gspace="post_skill_school_ianxu"
    gUrl="http://9.135.95.249:7001"

    taskId="12"

    importData(gUrl,taskId=taskId)


import networkx as nx
import tensorflow as tf
import random
import tqdm
import pandas as pd
import numpy as np
from maintain_PlatoUtils.maintain_PlatoUtils import wrapNebula2Df
from gensim.models import Word2Vec

import stellargraph as sg
from stellargraph import datasets,IndexedArray
from stellargraph.layer import GraphSAGE, link_classification, HinSAGE,DeepGraphInfomax
from sklearn.model_selection import train_test_split
from stellargraph.data import UnsupervisedSampler,BiasedRandomWalk
from stellargraph.mapper import Node2VecNodeGenerator,Node2VecLinkGenerator
from stellargraph.layer import Node2Vec

from nebula.graph import ttypes,GraphService
from nebula.ConnectionPool import ConnectionPool
from nebula.Client import GraphClient

def downloadGraph(gClient,subGraph=[{"head":{"type":"Company","keyAttr":"CompanyName"},
                                        "tail":{"type":"Field","keyAttr":"FieldName"},
                                        "edgeType":["belongTo"]}],space="post_skill_school_ianxu"):
    totalG=None
    if len(subGraph)>0:
        gClient.execute_query("USE {}".format(space))
        totalHtDfList=[]
        for triItem in subGraph:
            headType=triItem["head"]["type"]
            headKeyAttr=triItem["head"]["keyAttr"]
            tailType=triItem["tail"]["type"]
            tailKeyAttr=triItem["tail"]["keyAttr"]
            if len(triItem["edgeType"])>0:
                edgeTypeGroupStr=",".join(triItem["edgeType"])
            else:
                edgeTypeGroupStr="*"
            subGraphSearchStr="LOOKUP ON {headType} WHERE {headType}.{headKeyAttr}!='不可能的名字'|\
                                    GO FROM $-.VertexID OVER {edgeTypeGroup} YIELD \
                                        '{headType}_'+$^.{headType}.{headKeyAttr} AS srcName,\
                                        '{tailType}_'+$$.{tailType}.{tailKeyAttr} AS tgtName".format(
                                            headType=headType,headKeyAttr=headKeyAttr,
                                            tailType=tailType,tailKeyAttr=tailKeyAttr,
                                            edgeTypeGroup=edgeTypeGroupStr
                                        )
            
            htDfItem=wrapNebula2Df(gClient.execute_query(subGraphSearchStr))
            
            if htDfItem.shape[0]>0:
                htDfItem.dropna(inplace=True)
                totalHtDfList.append(htDfItem)
                
        if len(totalHtDfList)>0:
            totalHtDf=pd.concat(totalHtDfList)
            totalHtList=totalHtDf.values.tolist()
        
            random.seed(15)
            totalG=nx.from_edgelist(totalHtList)
            
    return totalG
    

def graphEmbedding(gClient,subGraph=[{"head":{"type":"Company","keyAttr":"CompanyName"},
                                        "tail":{"type":"Field","keyAttr":"FieldName"},
                                        "edgeType":["belongTo"]}],space="post_skill_school_ianxu",
                   model="DeepWalk",vecSize=128,totalG=None):
    if totalG is not None:
        totalG=downloadGraph(gClient,subGraph=subGraph,space=space)
    myW2VModel=None
    for nodeItem in totalG.nodes:
        totalG.nodes[nodeItem]["feature"]=np.random.random([vecSize,])
    if model=="DeepWalk":
        walk_number = 100
        walk_length = 15
        
        totalSGG=sg.StellarGraph.from_networkx(totalG, node_features="feature")
        walker = BiasedRandomWalk(
                    totalSGG,
                    n=walk_number,
                    length=walk_length,
                    p=1,q=1
                )
        walks = walker.run(totalSGG.nodes())
        
        myW2VModel=Word2Vec(walks,vector_size=vecSize,epochs=5)
            
    return myW2VModel

            
if __name__=="__main__":

    gHost="9.135.95.249"
    gPort=13708
    gUser="root"
    gPassword="nebula"
    gSpace="company_product_field_musklin"

    srcConnection_pool = ConnectionPool(gHost, gPort,network_timeout=300000)
    gClient = GraphClient(srcConnection_pool)
    gClient.authenticate(gUser, gPassword)
    gClient.execute_query("use {}".format(gSpace))

    myEmbModel=graphEmbedding(gClient,subGraph=[{"head":{"type":"Post","keyAttr":"PostName"},
                                                    "tail":{"type":"Skill","keyAttr":"SkillName"},
                                                    "edgeType":["contain"]}],
                                                space="post_skill_school_ianxu",model="DeepWalk",
                                                batch_size=128)
    
    print(123)
